///
/// @file
/// @details This scene is the gameplay scene for the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "gameplay_scene.h"
#include "scene_manager.h"
#include "score_display.h"
#include "game_results.h"
#include "win_game_scene.h"

#include "sliding_cards.h"
#include "card_entity.h"
#include "robot_entity.h"

#include "turtle_brains/graphics/unstable/tbu_gif_exporter.h"

std::vector<std::string> kLevels;

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::GameplayScene::GameplayScene(void)	:
	mPlayerPhase(PlayerTurnPhase::kGameStartup),
	mGameCondition(GameCondition::kPlayingGame),
	mWorld(*this),
	mConfirmButton(*this),
	mGameLoseBox(*this, false),
	mGameWinBox(*this, true),
	mCurrentLevel(0),
	mDrawCardTimer(0)
{
	mLeftClickAction.AddBinding(tbApplication::tbMouseLeft);
	mRightClickAction.AddBinding(tbApplication::tbMouseRight);
	mMiddleClickAction.AddBinding(tbApplication::tbMouseMiddle);

	mCameraMoveUpAction.AddBinding(tbApplication::tbKeyW);      mCameraMoveUpAction.AddBinding(tbApplication::tbKeyUp);
	mCameraMoveDownAction.AddBinding(tbApplication::tbKeyS);    mCameraMoveDownAction.AddBinding(tbApplication::tbKeyDown);
	mCameraMoveLeftAction.AddBinding(tbApplication::tbKeyA);    mCameraMoveLeftAction.AddBinding(tbApplication::tbKeyLeft);
	mCameraMoveRightAction.AddBinding(tbApplication::tbKeyD);   mCameraMoveRightAction.AddBinding(tbApplication::tbKeyRight);
	mCamereResetAction.AddBinding(tbApplication::tbKeySpace);   mCamereResetAction.AddBinding(tbApplication::tbMouseMiddle);

	mQuitGameAction.AddBinding(tbApplication::tbKeyEscape);

	AddGraphic(mWorld);
	AddEntity(mConfirmButton);
	mConfirmButton.SetDepth(0.15f);

	mGameLoseBox.CancelDoInput(); //Win box will do it.
	mGameLoseBox.SetDepth(0.15f);
	AddEntity(mGameLoseBox);

	mGameWinBox.SetDepth(0.15f);
	AddEntity(mGameWinBox);

	tbGraphics::Sprite* planner = new tbGraphics::Sprite("data/robot_planner.png");
	planner->SetScroll(0.0f);
	planner->SetOrigin(tbGraphics::kAnchorTopCenter);
	planner->SetPosition(tbGraphics::ScreenCenter().x, 0.0f);
	planner->SetDepth(0.05f);
	AddGraphic(planner);

	kLevels.push_back("data/levels/level_01.json");
	kLevels.push_back("data/levels/level_02.json");
	kLevels.push_back("data/levels/test_level.json");
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::GameplayScene::~GameplayScene(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::GameplayScene::IsAnyCardAnimating(bool checkOnlyTimer) const
{
	//return GetEntitiesByType("SlidingCardEntity").empty() ? false : true;
	if (true == checkOnlyTimer)
	{
		return !mDrawCardTimer.IsZero();
	}

	for (size_t cardIndex(0); cardIndex < mPlayerHand.Size(); ++cardIndex)
	{
		if (true == mPlayerHand.GetCardSlider(cardIndex)->IsAnimating())
		{
			return true;
		}
	}

	for (size_t cardIndex(0); cardIndex < mRobotPlanSlots.Size(); ++cardIndex)
	{
		if (true == mRobotPlanSlots.GetCardSlider(cardIndex)->IsAnimating())
		{
			return true;
		}
	}

	for (size_t cardIndex(0); cardIndex < mRobotPlanPile.Size(); ++cardIndex)
	{
		if (true == mRobotPlanPile.GetCardSlider(cardIndex)->IsAnimating())
		{
			return true;
		}
	}

	for (size_t cardIndex(0); cardIndex < mDiscardPile.Size(); ++cardIndex)
	{
		if (true == mDiscardPile.GetCardSlider(cardIndex)->IsAnimating())
		{
			return true;
		}
	}

	for (size_t cardIndex(0); cardIndex < mPunchCardDeck.Size(); ++cardIndex)
	{
		if (true == mPunchCardDeck.GetCardSlider(cardIndex)->IsAnimating())
		{
			return true;
		}
	}

	return false;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::OnConfirmRobotPlan(void)
{
	if (PlayerTurnPhase::kConfirmPlan == mPlayerPhase)
	{
		mConfirmButton.HideButton();
		mPlayerPhase = PlayerTurnPhase::kPowerUp;
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::OnFinishWinOrLose(void)
{
	if (GameCondition::kWonGame == mGameCondition)
	{
		++mCurrentLevel;
		if (mCurrentLevel >= kLevels.size())
		{
			LudumDare36::theSceneManager->ChangeToScene(kTitleScene);
			//mCurrentLevel = 0; //reset game.
			--mCurrentLevel; //Keep at last test level
		}
		else
		{
			LudumDare36::theSceneManager->ChangeToScene(kGameplayScene);
		}
	}
		
	if (GameCondition::kLostGame == mGameCondition)
	{	//Restart level
		LudumDare36::theSceneManager->ChangeToScene(kGameplayScene);
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::OnSimulate(void)
{
	tbGame::GameScene::OnSimulate();

	if (true == mQuitGameAction.IsReleased())
	{	//This should be the very last check to happen as game ends immediately.
		LudumDare36::theSceneManager->ChangeToScene(kTitleScene);
	}
	
	mWorld.Simulate();

	if (GameCondition::kPlayingGame != mGameCondition)
	{
		return;
	}

	mDrawCardTimer.DecrementStep();

	//Simulate the player phase...
	SimulatePlayerPhase();

	for (size_t cardIndex(0); cardIndex < mPlayerHand.Size(); ++cardIndex)
	{
		mPlayerHand.GetCardSlider(cardIndex)->Simulate();
	}

	for (size_t cardIndex(0); cardIndex < mRobotPlanSlots.Size(); ++cardIndex)
	{
		mRobotPlanSlots.GetCardSlider(cardIndex)->Simulate();
	}

	for (size_t cardIndex(0); cardIndex < mRobotPlanPile.Size(); ++cardIndex)
	{
		mRobotPlanPile.GetCardSlider(cardIndex)->Simulate();
	}
	
	for (size_t cardIndex(0); cardIndex < mDiscardPile.Size(); ++cardIndex)
	{
		mDiscardPile.GetCardSlider(cardIndex)->Simulate();
	}

	for (size_t cardIndex(0); cardIndex < mPunchCardDeck.Size(); ++cardIndex)
	{
		mPunchCardDeck.GetCardSlider(cardIndex)->Simulate();
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::SimulatePlayerPhase(void)
{
	int mouseX(0);
	int mouseY(0);
	tbGame::Input::GetMousePosition(&mouseX, &mouseY);
	const tbMath::Vector2 mousePosition(static_cast<float>(mouseX), static_cast<float>(mouseY));

	SimulateCamera(mousePosition);

	switch (mPlayerPhase)
	{
	case PlayerTurnPhase::kShuffleDiscard: {
		if (!IsAnyCardAnimating())
		{
			if (0 == mDiscardPile.Size())
			{
				if (!IsAnyCardAnimating(false))
				{	//Wait for all cards to stop shuffling.
					mDrawCardTimer = tbGame::GameTimer(500);
					mPunchCardDeck.ShuffleDeck();
					mPlayerPhase = PlayerTurnPhase::kDrawCards;
				}
			}
			else
			{
				mDrawCardTimer = tbGame::GameTimer(100);

				CardSlider* slider = mPunchCardDeck.AddCard(mDiscardPile.GetCard(mDiscardPile.Size() - 1));
				tb_error_if(nullptr == slider, "Error: This one was expected not null.");
				slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kDiscardTransform, SlidingCardEntity::kShuffleDeckTransform1, 500));
				slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kShuffleDeckTransform1, SlidingCardEntity::kShuffleDeckTransform2, 250));
				slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kShuffleDeckTransform2, SlidingCardEntity::kDeckTransform, 500));

				mDiscardPile.RemoveCard(mDiscardPile.Size() - 1);
			}
		}
		break; }
	case PlayerTurnPhase::kDrawCards: {
		if (!IsAnyCardAnimating())
		{
			if (mPlayerHand.Size() >= kMaximumHandCount)
			{
				mPlayerPhase = PlayerTurnPhase::kPlan;
				tb_log("Full Hand Drawn, moving to Planning Phase\n");
			}
			else
			{
				if (0 == mPunchCardDeck.Size())
				{
					tbAudio::theAudioManager.PlayEvent("audio_events", "shuffle_deck");
					mPlayerPhase = PlayerTurnPhase::kShuffleDiscard;
					tb_error_if(0 == mDiscardPile.Size(), "Expected discard pile to contain cards!");
				}
				else
				{
					mDrawCardTimer = tbGame::GameTimer(250);

					//tbAudio::theAudioManager.PlayEvent("audio_events", "draw_card");
					tbAudio::theAudioManager.PlayEvent("audio_events", "move_card");
					CardSlider* slider = mPlayerHand.AddCard(mPunchCardDeck.DrawCard());
					tb_error_if(nullptr == slider, "Error: This one was expected not null.");
					slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kDeckTransform, SlidingCardEntity::kHandTransform[mPlayerHand.Size() - 1], 500));
				}
			}
		}

		break; }
	case PlayerTurnPhase::kPlan: {
		if (mLeftClickAction.IsReleased())
		{
			for (size_t cardIndex(0); cardIndex < mPlayerHand.Size(); ++cardIndex)
			{
				if (true == SlidingCardEntity::IsPointContained(mousePosition, SlidingCardEntity::kHandTransform[cardIndex]))
				{
					tbAudio::theAudioManager.PlayEvent("audio_events", "choose_card");
					//tbAudio::theAudioManager.PlayEvent("audio_events", "move_card");
					CardSlider* slider = mRobotPlanSlots.AddCard(mPlayerHand.GetCard(cardIndex));
					tb_error_if(nullptr == slider, "Error: This one was expected not null.");
					slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kHandTransform[cardIndex], SlidingCardEntity::kSlotTransform[mRobotPlanSlots.Size() - 1], 500));

					mPlayerHand.RemoveCard(cardIndex);
					break;
				}
			}

			for (size_t cardIndex(0); cardIndex < mRobotPlanSlots.Size(); ++cardIndex)
			{
				if (true == SlidingCardEntity::IsPointContained(mousePosition, SlidingCardEntity::kSlotTransform[cardIndex]))
				{
					tbAudio::theAudioManager.PlayEvent("audio_events", "cancel_card");
					CardSlider* slider = mPlayerHand.AddCard(mRobotPlanSlots.GetCard(cardIndex));
					tb_error_if(nullptr == slider, "Error: This one was expected not null.");
					slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kSlotTransform[cardIndex], SlidingCardEntity::kHandTransform[mPlayerHand.Size() - 1], 500));

					mRobotPlanSlots.RemoveCard(cardIndex);
					break;
				}
			}
		}

		if (!IsAnyCardAnimating())
		{
			if (mRobotPlanSlots.Size() >= kMaximumSlotCount)
			{
				mPlayerPhase = PlayerTurnPhase::kConfirmPlan;
			}
		}

		break; }
	case PlayerTurnPhase::kConfirmPlan: {
		mConfirmButton.ShowButton();

		if (mLeftClickAction.IsReleased())
		{
			for (size_t cardIndex(0); cardIndex < mRobotPlanSlots.Size(); ++cardIndex)
			{
				if (true == SlidingCardEntity::IsPointContained(mousePosition, SlidingCardEntity::kSlotTransform[cardIndex]))
				{
					tbAudio::theAudioManager.PlayEvent("audio_events", "cancel_card");
					CardSlider* slider = mPlayerHand.AddCard(mRobotPlanSlots.GetCard(cardIndex));
					tb_error_if(nullptr == slider, "Error: This one was expected not null.");
					slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kSlotTransform[cardIndex], SlidingCardEntity::kHandTransform[mPlayerHand.Size() - 1], 500));

					mRobotPlanSlots.RemoveCard(cardIndex);

					mConfirmButton.HideButton();
					mPlayerPhase = PlayerTurnPhase::kPlan;
					break;
				}
			}

			//Phase will change to kPowerUp once player clicks Confirm Plan button, see OnConfirmRobotPlan().
		}

		break; }
	case PlayerTurnPhase::kPowerUp: {
		if (!IsAnyCardAnimating())
		{
			if (mPlayerHand.Size() > 0)
			{
				mDrawCardTimer = tbGame::GameTimer(250); //DiscardTimer

				tbAudio::theAudioManager.PlayEvent("audio_events", "discard_card");
				CardSlider* slider = mDiscardPile.AddCard(mPlayerHand.GetCard(mPlayerHand.Size() - 1));
				tb_error_if(nullptr == slider, "Error: This one was expected not null.");
				slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kHandTransform[mPlayerHand.Size() - 1], SlidingCardEntity::kDiscardTransform, 500));

				mPlayerHand.RemoveCard(mPlayerHand.Size() - 1);
			}
			else if (mRobotPlanSlots.Size() > 0)
			{
				mDrawCardTimer = tbGame::GameTimer(500); //DiscardTimer

				tbAudio::theAudioManager.PlayEvent("audio_events", "move_card");
				CardSlider* slider = mRobotPlanPile.AddCard(mRobotPlanSlots.GetCard(mRobotPlanSlots.Size() - 1));
				tb_error_if(nullptr == slider, "Error: This one was expected not null.");
				slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kSlotTransform[mRobotPlanSlots.Size() - 1], SlidingCardEntity::kRobotPlanTransform, 500));

				mRobotPlanSlots.RemoveCard(mRobotPlanSlots.Size() - 1);
			}
			else
			{
				mPlayerPhase = PlayerTurnPhase::kRunRobot;
				mGamePhase = GameTurnPhase::kBeginRound;
			}
		}

		break; }
	case PlayerTurnPhase::kRunRobot: {
		SimulateGamePhase();
		if (GameTurnPhase::kEndRound == mGamePhase)
		{
			mPlayerPhase = PlayerTurnPhase::kPowerDown;
		}
		break; }
	case PlayerTurnPhase::kPowerDown: {
		//Play sound, give feedback, go draw cards.
		//TODO: LD36: check win condition

		mPlayerPhase = PlayerTurnPhase::kDrawCards;
		break; }
	};
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::SimulateGamePhase(void)
{
	switch (mGamePhase)
	{
	case GameTurnPhase::kBeginRound: {
		mGamePhase = GameTurnPhase::kRobotBegin;
		tbAudio::theAudioManager.PlayEvent("audio_events", "round_begin");
		break; }

	case GameTurnPhase::kRobotBegin: {
		tbAudio::theAudioManager.PlayEvent("audio_events", "robot_move");
		mWorld.BeginRobotTurn(mRobotPlanPile.GetCard(mRobotPlanPile.Size() - 1));
		mGamePhase = GameTurnPhase::kRobotTurn;
		break; }
	case GameTurnPhase::kRobotTurn: {
		if (true == mWorld.SimulateRobotTurn())
		{
			if (false == mWorld.GetRobot().IsAlive())
			{	//This code is duplicated down in the EnvironmentTurn (below) as well ...
				tb_log("Robot has fallen or died with instant kill, or hit points.\n");
				//WinGameScene::sWinResult = false;
				//SceneManager::ChangeToScene(LudumDare36::kGameWinScene);
				mGameCondition = GameCondition::kLostGame;
				mGameLoseBox.ShowButton();
				tbAudio::theAudioManager.PlayEvent("audio_events", "game_lose");
			}

			mGamePhase = GameTurnPhase::kEnvironmentBegin;
		}
		break; }

	case GameTurnPhase::kEnvironmentBegin: {
		mWorld.BeginEnvironmentTurn();
		mGamePhase = GameTurnPhase::kEnvironmentTurn;
		break; }
	case GameTurnPhase::kEnvironmentTurn: {
		if (true == mWorld.SimulateEnvironmentTurn())
		{
			tbAudio::theAudioManager.PlayEvent("audio_events", "discard_card");
			CardSlider* slider = mDiscardPile.AddCard(mRobotPlanPile.GetCard(mRobotPlanPile.Size() - 1));
			tb_error_if(nullptr == slider, "Error: This one was expected not null.");
			slider->PushEffect(CardSlidingEffect(SlidingCardEntity::kRobotPlanTransform, SlidingCardEntity::kDiscardTransform, 500));
			mRobotPlanPile.RemoveCard(mRobotPlanPile.Size() - 1);

			if (false == mWorld.GetRobot().IsAlive())
			{	//This code is duplicated down in the RobotTurn (above) as well ...
				tb_log("Robot has fallen or died with instant kill, or hit points.\n");
				//WinGameScene::sWinResult = false;
				//SceneManager::ChangeToScene(LudumDare36::kGameWinScene);
				mGameCondition = GameCondition::kLostGame;
				mGameLoseBox.ShowButton();
				tbAudio::theAudioManager.PlayEvent("audio_events", "game_lose");
			}

//#define do_win_per_round
#if defined(do_win_per_round)
			if (mRobotPlanPile.Size() > 0)
			{
				mGamePhase = GameTurnPhase::kRobotBegin;
			}
			else
			{
				mGamePhase = GameTurnPhase::kWinCheck;
			}
#else
			mGamePhase = GameTurnPhase::kWinCheck;
#endif
		}
		break; }

	case GameTurnPhase::kWinCheck: {
		if (true == mWorld.IsRobotAtHome())
		{
			tb_log("Robot is home WIN CHECK\n");
			//WinGameScene::sWinResult = true;
			//SceneManager::ChangeToScene(LudumDare36::kGameWinScene);
			mGameCondition = GameCondition::kWonGame;
			mGameWinBox.ShowButton();

			tbAudio::theAudioManager.PlayEvent("audio_events", "game_win");
		}
		else
		{
			tbAudio::theAudioManager.PlayEvent("audio_events", "round_complete");
		}

#if defined(do_win_per_round)
		mGamePhase = GameTurnPhase::kEndRound;
#else
		if (mRobotPlanPile.Size() > 0)
		{
			mGamePhase = GameTurnPhase::kRobotBegin;
		}
		else
		{
			mGamePhase = GameTurnPhase::kEndRound;
		}

#endif

		break; }
	};
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::ResetCameraPosition(void)
{
	mCameraPosition = mWorld.GetRobot().GetPosition();
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::SimulateCamera(const tbMath::Vector2& mousePosition)
{
	const float kMovementRate(256.0f);
	if (true == mCameraMoveUpAction.IsDown()) { mCameraPosition.y -= kMovementRate * kFixedTime; }
	if (true == mCameraMoveDownAction.IsDown()) { mCameraPosition.y += kMovementRate * kFixedTime; }
	if (true == mCameraMoveLeftAction.IsDown()) { mCameraPosition.x -= kMovementRate * kFixedTime; }
	if (true == mCameraMoveRightAction.IsDown()) { mCameraPosition.x += kMovementRate * kFixedTime; }

	if (true == mCamereResetAction.IsPressed()) { ResetCameraPosition(); }

	if (true == mRightClickAction.IsPressed())
	{
		mCameraDragDown = mousePosition;
	}
	else if (true == mRightClickAction.IsDown())
	{
		mCameraPosition += mCameraDragDown - mousePosition;
		mCameraDragDown = mousePosition;
	}

	if (PlayerTurnPhase::kRunRobot == mPlayerPhase)
	{
		ResetCameraPosition();
	}

	tbGraphics::GetActiveCamera().SetPosition(mCameraPosition - tbGraphics::ScreenCenter());
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::OnUpdate(const float deltaTime)
{
	tbGame::GameScene::OnUpdate(deltaTime);

	tbMath::Vector2 cameraPosition(tbGraphics::GetActiveCamera().GetPosition());
	cameraPosition.x = std::round(cameraPosition.x);
	cameraPosition.y = std::round(cameraPosition.y);
	tbGraphics::GetActiveCamera().SetPosition(cameraPosition);

	tbGraphics::Unstable::GifPerformCapture(deltaTime, tbGame::Input::IsKeyPressed(tbApplication::tbKeyG));
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::OnRender(void) const
{
	SlidingCardEntity::RenderCard(PunchCard::kDeckPile, SlidingCardEntity::kDeckTransform);
	SlidingCardEntity::RenderCard(PunchCard::kDiscardPile , SlidingCardEntity::kDiscardTransform);

	for (size_t cardIndex(0); cardIndex < mPlayerHand.Size(); ++cardIndex)
	{
		SlidingCardEntity::RenderCard(mPlayerHand.GetCard(cardIndex), mPlayerHand.GetCardSlider(cardIndex)->GetCardTransform());
	}

	for (int cardIndex(LudumDare36::kMaximumSlotCount - 1); cardIndex >= 0; --cardIndex)
	{
		if (static_cast<size_t>(cardIndex) < mRobotPlanSlots.Size())
		{
			SlidingCardEntity::RenderCard(mRobotPlanSlots.GetCard(cardIndex), mRobotPlanSlots.GetCardSlider(cardIndex)->GetCardTransform());
		}
		else
		{
			SlidingCardEntity::RenderCard(PunchCard::kEmptyPile, SlidingCardEntity::kSlotTransform[cardIndex]);
		}
	}

	for (size_t cardIndex(0); cardIndex < mRobotPlanPile.Size(); ++cardIndex)
	{
		SlidingCardEntity::RenderCard(mRobotPlanPile.GetCard(cardIndex), mRobotPlanPile.GetCardSlider(cardIndex)->GetCardTransform());
	}	

	for (size_t cardIndex(0); cardIndex < mDiscardPile.Size(); ++cardIndex)
	{
		SlidingCardEntity::RenderCard(mDiscardPile.GetCard(cardIndex), mDiscardPile.GetCardSlider(cardIndex)->GetCardTransform());
	}

	if (PlayerTurnPhase::kShuffleDiscard == mPlayerPhase)
	{
		for (size_t cardIndex(0); cardIndex < mPunchCardDeck.Size(); ++cardIndex)
		{
			SlidingCardEntity::RenderCard(mPunchCardDeck.GetCard(cardIndex), mPunchCardDeck.GetCardSlider(cardIndex)->GetCardTransform());
		}
	}

	tbGame::GameScene::OnRender();
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::OnOpen(void)
{
	tbGame::GameScene::OnOpen();

	tbMath::SeedRandomGenerator(0);
	
	mPlayerPhase = PlayerTurnPhase::kGameStartup;
	mGameCondition = GameCondition::kPlayingGame;
	mGamePhase = GameTurnPhase::kBeginRound;

	mPunchCardDeck.CreateNewDeck();
	mPunchCardDeck.ShuffleDeck();

	mPlayerHand.ClearCards();
	mRobotPlanSlots.ClearCards();
	mDiscardPile.ClearCards();
	mRobotPlanPile.ClearCards();

	//mWorld.LoadWorld("data/levels/test_level.json");

	std::string levelToLoad(kLevels[mCurrentLevel]);
	tb_log("Loading level %d: file: %s\n", mCurrentLevel, levelToLoad.c_str());
	mWorld.LoadWorld(levelToLoad);
	ResetCameraPosition();

	mGameLoseBox.HideButton(true);
	mGameWinBox.HideButton(true);
	mConfirmButton.HideButton(true);

	mPlayerPhase = PlayerTurnPhase::kDrawCards;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GameplayScene::OnClose(void)
{
	tbGame::GameScene::OnClose();
}

//--------------------------------------------------------------------------------------------------------------------//
