///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "robot_entity.h"
#include "world_environment.h"

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::GamePiece::GamePiece(void) :
	tbGame::Entity("GamePiece"),
	mSprite("robot_sheet", "robot")
{
	mSprite.PlayAnimation("idle", true, true, 1.0f / 10.0f);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::GamePiece::~GamePiece(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::SetWorldTransform(const WorldTransform& worldTransform)
{
	mWorldTransform = worldTransform;
	SetPosition(mWorldTransform.mPosition);
	mSprite.SetRotation(mWorldTransform.mOrientation);
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::ResetGame(void)
{
	OnResetGame();
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::BeginTurn(const WorldTransform& turnResult)
{
	mTurnResultTransform = turnResult;
	OnBeginTurn(turnResult);
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::SimulateTurn(float turnPercentage)
{
	SetPosition(tbMath::Interpolation::Linear(turnPercentage, mWorldTransform.mPosition, mTurnResultTransform.mPosition));
	mSprite.SetRotation(tbMath::Interpolation::Linear(turnPercentage, mWorldTransform.mOrientation, mTurnResultTransform.mOrientation));

	OnSimulateTurn(turnPercentage);
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::EndTurn(void)
{
	SetWorldTransform(mTurnResultTransform);
	OnEndTurn();
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::OnResetGame()
{
	mTurnResultTransform = mWorldTransform;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::OnBeginTurn(const WorldTransform& turnResult)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::OnSimulateTurn(float turnPercentage)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::GamePiece::OnEndTurn(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::RobotEntity::RobotEntity(const WorldEnvironment& worldEnvironment) :
	mWorldEnvironment(worldEnvironment),
	mShieldBurstSprite("robot_sheet", "shield_burst"),
	mHealth(HealthDisplayEntity::kMaximumHealth)
{
	AddEntityType("RobotEntity");

	SetPosition(mWorldTransform.mPosition);

	mSprite.SetOrigin(tbGraphics::kAnchorCenter);
	AddGraphic(mSprite);

	mShieldBurstSprite.SetOrigin(tbGraphics::kAnchorCenter);
	mShieldBurstSprite.PlayAnimation("no_burst");
	AddGraphic(mShieldBurstSprite);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::RobotEntity::~RobotEntity(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::RobotEntity::HurtRobot(const int amount)
{
	tbAudio::theAudioManager.PlayEvent("audio_events", "robot_hurt");
	mShieldBurstSprite.PlayAnimation("burst");
	tbGraphics::GetActiveCamera().ShakeCamera(tbGraphics::Camera::kMediumShaking);
	mHealth -= amount;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::RobotEntity::PerformInstantKill(void)
{
	tbAudio::theAudioManager.PlayEvent("audio_events", "robot_instant_kill");
	mHealth = 0;
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::RobotEntity::IsAlive(void) const
{
	return mHealth > 0;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::RobotEntity::OnResetGame(void)
{
	mShieldBurstSprite.PlayAnimation("no_burst");
	mHealth = HealthDisplayEntity::kMaximumHealth;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::RobotEntity::OnBeginTurn(const WorldTransform& turnResult)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::RobotEntity::OnEndTurn(void)
{
	mWorldEnvironment.PerformInstantKillEffects(*this);
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::RobotEntity::OnAdded(void)
{
	GetEntityManager()->AddEntity(new HealthDisplayEntity(*this));
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::RobotEntity::OnCollideWith(tbGame::Entity& otherEntity)
{
	tbGame::Entity::OnCollideWith(otherEntity);
}

//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------////--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::HealthDisplayEntity::HealthDisplayEntity(const RobotEntity& robot) :
	tbGame::Entity("HealthDisplayEntity"),
	mRobot(robot),
	mLastHealth(robot.GetHealth()),
	mHealthSprites{tbGraphics::AnimatedSprite("robot_sheet", "shield"), tbGraphics::AnimatedSprite("robot_sheet", "shield"), 
		tbGraphics::AnimatedSprite("robot_sheet", "shield"), tbGraphics::AnimatedSprite("robot_sheet", "shield") }
{
	tbMath::Vector2 position(560.0f, 94.0f);
	for (int index = 0; index < kMaximumHealth; ++index)
	{
		mHealthSprites[index].SetPosition(position + tbMath::Vector2(40.0f * index, 0.0f));
		mHealthSprites[index].SetScroll(0.0f);
		mHealthSprites[index].SetDepth(0.09f);

		mHealthSprites[index].PlayAnimation("good");

		AddGraphic(mHealthSprites[index]);
	}
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::HealthDisplayEntity::~HealthDisplayEntity(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::HealthDisplayEntity::OnSimulate(void)
{
	const int currentHealth(mRobot.GetHealth());
	if (currentHealth != mLastHealth)
	{
		while (currentHealth > mLastHealth)
		{
			++mLastHealth;
			if (mLastHealth < kMaximumHealth)
			{
				mHealthSprites[mLastHealth].PlayAnimation("good");
			}
		}

		while (currentHealth < mLastHealth)
		{
			--mLastHealth;
			if (mLastHealth >= 0)
			{
				mHealthSprites[mLastHealth].PlayAnimation("hurt");
			}
		}
	}
}

//--------------------------------------------------------------------------------------------------------------------//
