///
/// @file
/// @details This scene is the gameplay scene for the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "splash_scene.h"
#include "scene_manager.h"
#include "score_display.h"
#include "game_results.h"

const tbGame::GameTimer kFinishFadeInTime(1000);
const tbGame::GameTimer kStartFadeOutTime(3500);
const tbGame::GameTimer kMaximumSplashTime(4000);

const tbGame::GameTimer kMaximumFadeInTime(500);
const tbGame::GameTimer kMaximumFadeOutTime(500);

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::SplashScene::SplashScene(void) :
	mSceneTimer(0),
	mBlackbirdLogo("data/blackbird_logo_512.png")
{
	mJumpToGameAction.AddBinding(tbApplication::tbKeyEscape);
	mJumpToGameAction.AddBinding(tbApplication::tbKeySpace);
	mJumpToGameAction.AddBinding(tbApplication::tbMouseLeft);

	mBlackbirdLogo.SetScale(0.5f);
	mBlackbirdLogo.SetColor(tbGraphics::Color(0x00FFFFFF));
	mBlackbirdLogo.SetOrigin(tbGraphics::kAnchorCenter);
	mBlackbirdLogo.SetPosition(tbGraphics::ScreenCenter());
	AddGraphic(mBlackbirdLogo);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::SplashScene::~SplashScene(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::SplashScene::OnSimulate(void)
{
	tbGame::GameScene::OnSimulate();

	if (true == mJumpToGameAction.IsReleased() && mSceneTimer < kStartFadeOutTime)
	{
		mSceneTimer = kStartFadeOutTime;
	}

	if (mSceneTimer.IncrementStep(kMaximumSplashTime))
	{
		theSceneManager->ChangeToScene(SceneId::kTitleScene);
	}

	if (mSceneTimer.GetElapsedTime() < kFinishFadeInTime.GetElapsedTime())
	{	//Fade the logo in.
		float percentage = mSceneTimer.GetPercentageOf(kFinishFadeInTime);
		tbGraphics::Color fadingColor;
		fadingColor.SetColor(percentage, tbGraphics::Color(0x00FFFFFF), tbGraphics::Color(0xFFFFFFFF));
		mBlackbirdLogo.SetColor(fadingColor);
	}
	else if (mSceneTimer.GetElapsedTime() < kStartFadeOutTime.GetElapsedTime())
	{
		mBlackbirdLogo.SetColor(tbGraphics::Color::kWhite);
	}
	else
	{	//Fade the logo out.
		mBlackbirdLogo.SetColor(tbGraphics::Color(0x00FFFFFF));

		const float currentTiming = static_cast<float>(mSceneTimer.GetElapsedTime() - kStartFadeOutTime.GetElapsedTime());
		const float timingRange = static_cast<float>(kMaximumSplashTime.GetElapsedTime() - kStartFadeOutTime.GetElapsedTime());
		const float percentage = (currentTiming / timingRange);
		tb_log("percentage: %.2f   time: %.2f    range: %.2f\n", percentage, currentTiming, timingRange);
		tbGraphics::Color fadingColor;
		fadingColor.SetColor(percentage, tbGraphics::Color(0xFFFFFFFF), tbGraphics::Color(0x00FFFFFF));
		mBlackbirdLogo.SetColor(fadingColor);
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::SplashScene::OnUpdate(const float deltaTime)
{
	tbGame::GameScene::OnUpdate(deltaTime);
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::SplashScene::OnRender(void) const
{
	tbGame::GameScene::OnRender();
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::SplashScene::OnOpen(void)
{
	tbGame::GameScene::OnOpen();

	mSceneTimer = tbGame::GameTimer::kZero;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::SplashScene::OnClose(void)
{
	tbGame::GameScene::OnClose();
}

//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//
//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::TitleScene::TitleScene(void) :
	mTitleScreen("data/title_screen.png"),
	mFadeInTimer(kMaximumFadeInTime),
	mFadeOutTimer(0)
{
	mTitleScreen.SetPosition(0.0f, 0.0f);
	mTitleScreen.SetColor(tbGraphics::Color(0x00FFFFFF));
	AddGraphic(mTitleScreen);

	mStartGameAction.AddBinding(tbApplication::tbKeyEscape);
	mStartGameAction.AddBinding(tbApplication::tbKeySpace);
	mStartGameAction.AddBinding(tbApplication::tbMouseLeft);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::TitleScene::~TitleScene(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::TitleScene::OnSimulate(void)
{
	if (false == mFadeInTimer.IsZero())
	{
		float percentage = mFadeInTimer.GetPercentageOf(kMaximumFadeInTime);
		tbGraphics::Color fadingColor;
		fadingColor.SetColor(percentage, tbGraphics::Color(0xFFFFFFFF), tbGraphics::Color(0x00FFFFFF));
		mTitleScreen.SetColor(fadingColor);

		if (true == mFadeInTimer.DecrementStep())
		{
			mTitleScreen.SetColor(tbGraphics::Color(0xFFFFFFFF));
		}
	}

	if (false == mFadeOutTimer.IsZero())
	{
		float percentage = mFadeOutTimer.GetPercentageOf(kMaximumFadeOutTime);
		tbGraphics::Color fadingColor;
		fadingColor.SetColor(percentage, tbGraphics::Color(0x00FFFFFF), tbGraphics::Color(0xFFFFFFFF));
		mTitleScreen.SetColor(fadingColor);

		if (true == mFadeOutTimer.DecrementStep())
		{
			mTitleScreen.SetColor(tbGraphics::Color(0x00FFFFFF));
			theSceneManager->ChangeToScene(SceneId::kGameplayScene);
		}
	}
	else
	{
		if (true == mStartGameAction.IsPressed())
		{
			mFadeOutTimer = kMaximumFadeOutTime;
		}
	}

	tbGame::GameScene::OnSimulate();
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::TitleScene::OnOpen(void)
{
	mFadeInTimer = kMaximumFadeInTime;
	mFadeOutTimer = 0;
	mTitleScreen.SetColor(tbGraphics::Color(0x00FFFFFF));
}

//--------------------------------------------------------------------------------------------------------------------//

