///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "world_environment.h"

//--------------------------------------------------------------------------------------------------------------------//

const size_t LudumDare36::WorldEnvironment::kWorldTileSize(64);

const tbGame::GameTimer LudumDare36::WorldEnvironment::kMaximumTurnExecutionTime(750);
const tbGame::GameTimer LudumDare36::WorldEnvironment::kMinimumDelayBetweenTurns(500);

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldEnvironment::WorldEnvironment(tbGame::EntityManager& entityManager) :
	mEntityManager(entityManager),
	mRobot(*this),
	mTurnDelayTimer(tbGame::GameTimer::kZero),
	mEnvironmentTurnTimer(tbGame::GameTimer::kZero),
	mRobotTurnTimer(tbGame::GameTimer::kZero),
	mRobotHurtTimer(tbGame::GameTimer::kZero)
{
	mTileSystem.SetDepth(-0.5f);
	AddGraphic(mTileSystem);
	
	mEntityManager.AddEntity(mRobot);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldEnvironment::~WorldEnvironment(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::WorldEnvironment::LoadWorld(const tbCore::tbString& levelPath)
{
	//First clear out the old world / reset the game.
	mTileSystem.ClearMap();
	mRobot.ResetGame();;

	mTurnDelayTimer = tbGame::GameTimer::kZero;
	mEnvironmentTurnTimer = tbGame::GameTimer::kZero;
	mRobotTurnTimer = tbGame::GameTimer::kZero;
	mRobotHurtTimer = tbGame::GameTimer::kZero;

	mHomePoints.clear();
	mImpassibleTiles.clear();
	mInstantKillTiles.clear();
	for (size_t index(0); index < 4; ++index)
	{
		mSlidingTiles[index].clear();
	}



	//TODO: TIM: TurtleBrains: This seems to always return false, fix it after LD36
	tbGame::Extensions::LoadTileSystemFromTiled(mTileSystem, levelPath);

	//Find the start tile.
	tbGame::AboutTileContainer startTiles = mTileSystem.FindTilesWithPropertyEquals("startTile", true);
	tb_error_if(1 != startTiles.size(), "Error: Expected 1 tile with property startTile to be true.");
	const tbGame::AboutTile& startTile(startTiles[0]);
	mRobot.SetWorldTransform(WorldTransform(startTile.mColumn, startTile.mRow));
	mTileSystem.SetTile(startTile.mLayerName, startTile.mColumn, startTile.mRow, 0);

	//Find the home tile(s)
	tbGame::AboutTileContainer homeTiles = mTileSystem.FindTilesWithPropertyEquals("homeTile", true);
	tb_error_if(1 > homeTiles.size(), "Error: Expected to find at least 1 tile with property homeTile to be true.");
	for (const tbGame::AboutTile& aboutTile : homeTiles)
	{
		mHomePoints.push_back(WorldTransform(aboutTile.mColumn, aboutTile.mRow));
	}

	//Find any tiles that block movement
	tbGame::AboutTileContainer impassableTiles = mTileSystem.FindTilesWithPropertyEquals("impassableTile", true);
	for (const tbGame::AboutTile& aboutTile : impassableTiles)
	{
		mImpassibleTiles.push_back(WorldTransform(aboutTile.mColumn, aboutTile.mRow));
	}

	//Find any tiles that slide the robot around, like streams
	tbGame::AboutTileContainer slidingTiles = mTileSystem.FindTilesWithPropertyEquals("slidingTile", true);
	for (const tbGame::AboutTile& aboutTile : slidingTiles)
	{
		int index = mTileSystem.GetTilePropertyAsInteger(aboutTile.mTileSetName, aboutTile.mTileIndex, "direction");
		mSlidingTiles[index].push_back(WorldTransform(aboutTile.mColumn, aboutTile.mRow));
	}

	//Find any tiles that kill the robot when they enter. (falling for instance).
	tbGame::AboutTileContainer instantKillTiles = mTileSystem.FindTilesWithPropertyEquals("instantKillTile", true);
	for (const tbGame::AboutTile& aboutTile : instantKillTiles)
	{
		mInstantKillTiles.push_back(WorldTransform(aboutTile.mColumn, aboutTile.mRow));
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::WorldEnvironment::Simulate(void)
{
	AnimateWater();
	tbGraphics::GetActiveCamera().SetPosition(mRobot.GetPosition() - tbGraphics::ScreenCenter());

	if (false == mRobotHurtTimer.IsZero())
	{
		if (true == mRobotHurtTimer.DecrementStep())
		{	//We got hurt not long ago.
			mRobot.HurtRobot(1);
		}
	}
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldTransform LudumDare36::WorldEnvironment::GetTransformForInstruction(const WorldTransform& inputTransform, const PunchCard& punchCard)
{
	WorldTransform nextTransform(inputTransform);
	switch (punchCard)
	{
	case PunchCard::kMoveForward1: nextTransform = WorldTransform::MoveForward(inputTransform, 1); break;
	case PunchCard::kMoveForward2: nextTransform = WorldTransform::MoveForward(inputTransform, 2); break;
	case PunchCard::kMoveBackward1: nextTransform = WorldTransform::MoveForward(inputTransform, -1); break;
	case PunchCard::kMoveBackward2: nextTransform = WorldTransform::MoveForward(inputTransform, -2); break;

	case PunchCard::kTurn90Left: nextTransform = WorldTransform::Rotate90Left(inputTransform); break;
	case PunchCard::kTurn90Right: nextTransform = WorldTransform::Rotate90Right(inputTransform); break;
	case PunchCard::kTurn180: nextTransform = WorldTransform::Rotate180(inputTransform); break;
	};

	return nextTransform;
}

//--------------------------------------------------------------------------------------------------------------------//

std::vector<LudumDare36::WorldTransform> LudumDare36::WorldEnvironment::GetAllTransformsForInstruction(const WorldTransform& input, const PunchCard& punchCard)
{
	std::vector<WorldTransform> allTransforms;
	allTransforms.push_back(GetTransformForInstruction(input, punchCard));

	switch (punchCard)
	{
	case PunchCard::kMoveForward2: allTransforms.push_back(GetTransformForInstruction(input, PunchCard::kMoveForward1)); break;
	case PunchCard::kMoveBackward2: allTransforms.push_back(GetTransformForInstruction(input, PunchCard::kMoveBackward1)); break;
	}

	return allTransforms;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::WorldEnvironment::BeginEnvironmentTurn(void)
{
	mTurnDelayTimer = kMinimumDelayBetweenTurns;
	mEnvironmentTurnTimer = kMaximumTurnExecutionTime;

	const WorldTransform& robotTransform(mRobot.GetWorldTransform());
	WorldTransform robotTurnResult = robotTransform;

	WorldDirection directions[] = { WorldDirection::kNorth, WorldDirection::kEast, WorldDirection::kSouth, WorldDirection::kWest };

	for (size_t index(0); index < 4; ++index)
	{
		for (const WorldTransform& slidingTransform : mSlidingTiles[index])
		{
			if (slidingTransform.mColumn == robotTransform.mColumn && slidingTransform.mRow == robotTransform.mRow)
			{
				tbAudio::theAudioManager.PlayEvent("audio_events", "water_move");

				WorldTransform newLocation(WorldTransform::MoveWorld(robotTransform, directions[index], 1));
				if (true == CanMoveToLocation(mRobot, newLocation))
				{
					robotTurnResult = newLocation;
				}
				else
				{
					mRobotHurtTimer = 250; //mRobot.HurtRobot(1);
				}
			}
		}
	}

	mRobot.BeginTurn(robotTurnResult);
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::WorldEnvironment::SimulateEnvironmentTurn(void)
{
	if (false == mTurnDelayTimer.IsZero())
	{
		mTurnDelayTimer.DecrementStep();
		return false;
	}

	if (true == mEnvironmentTurnTimer.DecrementStep())
	{	//For each of the GamePieces in the world, including robot.
		mRobot.EndTurn();
		return true;
	}

	//For each of the GamePieces in the world, including robot.
	const float turnPercentage(1.0f - mEnvironmentTurnTimer.GetPercentageOf(kMaximumTurnExecutionTime));
	mRobot.SimulateTurn(turnPercentage);
	return false;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::WorldEnvironment::BeginRobotTurn(const PunchCard& punchCard)
{
	mTurnDelayTimer = kMinimumDelayBetweenTurns;
	mRobotTurnTimer = kMaximumTurnExecutionTime;

	WorldTransform robotTurnResult = WorldEnvironment::GetTransformForInstruction(mRobot.GetWorldTransform(), punchCard);
	if (false == CanPerformInstruction(mRobot, punchCard))
	{
		mRobotHurtTimer = 250; //mRobot.HurtRobot(1);
		robotTurnResult = mRobot.GetWorldTransform();

		if (PunchCard::kMoveBackward2 == punchCard)
		{
			if (true == CanPerformInstruction(mRobot, PunchCard::kMoveBackward1))
			{
				robotTurnResult = WorldEnvironment::GetTransformForInstruction(mRobot.GetWorldTransform(), PunchCard::kMoveBackward1);
				mRobotHurtTimer = 500; //We still get hurt by a rock.
			}			
		}
		else if (PunchCard::kMoveForward2 == punchCard)
		{
			if (true == CanPerformInstruction(mRobot, PunchCard::kMoveForward1))
			{
				robotTurnResult = WorldEnvironment::GetTransformForInstruction(mRobot.GetWorldTransform(), PunchCard::kMoveForward1);
				mRobotHurtTimer = 500; //We still get hurt by a rock.
			}
		}
	}

	mRobot.BeginTurn(robotTurnResult);
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::WorldEnvironment::SimulateRobotTurn(void)
{
	if (false == mTurnDelayTimer.IsZero())
	{
		mTurnDelayTimer.DecrementStep();
		return false;
	}

	if (true == mRobotTurnTimer.DecrementStep())
	{
		mRobot.EndTurn();
		return true;
	}

	const float turnPercentage(1.0f - mRobotTurnTimer.GetPercentageOf(kMaximumTurnExecutionTime));
	mRobot.SimulateTurn(turnPercentage);
	return false;
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::WorldEnvironment::CanPerformInstruction(const RobotEntity& robot, const PunchCard& punchCard) const
{
	std::vector<WorldTransform> allTransforms = WorldEnvironment::GetAllTransformsForInstruction(robot.GetWorldTransform(), punchCard);
	
	WorldEnvironment::GetTransformForInstruction(robot.GetWorldTransform(), punchCard);

	for (const WorldTransform& transform : allTransforms)
	{
		if (false == CanMoveToLocation(robot, transform))
		{
			return false;
		}
	}

	return true;
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::WorldEnvironment::CanMoveToLocation(const RobotEntity& robot, const WorldTransform& transform) const
{
	for (const WorldTransform& impassableTransform : mImpassibleTiles)
	{
		if (impassableTransform.mColumn == transform.mColumn && impassableTransform.mRow == transform.mRow)
		{	//This hurts the robot.
			return false;
		}
	}

	return true;
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::WorldEnvironment::PerformInstantKillEffects(RobotEntity& robot) const
{
	const WorldTransform& robotTransform(robot.GetWorldTransform());

	for (const WorldTransform& instantKillTransform : mInstantKillTiles)
	{
		if (instantKillTransform.mColumn == robotTransform.mColumn && instantKillTransform.mRow == robotTransform.mRow)
		{	//This kills the robot.
			robot.PerformInstantKill();
			return true;
		}
	}

	return false;
}

//--------------------------------------------------------------------------------------------------------------------//

bool LudumDare36::WorldEnvironment::IsRobotAtHome(void) const
{
	const WorldTransform& robotTransform(mRobot.GetWorldTransform());

	for (const WorldTransform& homeTransform : mHomePoints)
	{
		if (homeTransform.mColumn == robotTransform.mColumn && homeTransform.mRow == robotTransform.mRow)
		{
			return true;
		}
	}

	return false;
}

//--------------------------------------------------------------------------------------------------------------------//

void LudumDare36::WorldEnvironment::AnimateWater(void)
{
	static int counter = 0;
	counter += 10;

	static int current = 0;

	static const tbGame::TileIndex waterFlow[4][7] = {
		19, 35, 51, 67, 83, 99, 115,
		114, 98, 82, 66, 50, 34, 18,
		113, 97, 81, 65, 49, 33, 17,
		20, 36, 52, 68, 84, 100, 116,
	};

	if (counter > 80)
	{
		counter = 0;
		++current;
		if (current >= 7)
		{
			current = 0;
		}

		for (int directionIndex = 0; directionIndex < 4; ++directionIndex)
		{
			tbGame::AboutTileContainer slidingTiles = mTileSystem.FindTilesWithPropertyEquals("direction", directionIndex);
			for (const tbGame::AboutTile& aboutTile : slidingTiles)
			{
				mTileSystem.SetTile(aboutTile.mLayerName, aboutTile.mColumn, aboutTile.mRow, waterFlow[directionIndex][current]);
			}
		}
	}
}

//--------------------------------------------------------------------------------------------------------------------//
