///
/// @file
/// @details An entity within the LudumDare36 project.
///
/// <!-- Copyright (c) 2016 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "world_transform.h"
#include "world_environment.h"

//--------------------------------------------------------------------------------------------------------------------//

tbMath::Vector2 LudumDare36::DirectionToVector(const LudumDare36::WorldDirection& direction)
{
	switch (direction)
	{
	case WorldDirection::kNorth: return tbMath::Vector2(0.0f, -1.0f);
	case WorldDirection::kEast: return tbMath::Vector2(1.0f, 0.0f);
	case WorldDirection::kSouth: return tbMath::Vector2(0.0f, 1.0f);
	case WorldDirection::kWest: return tbMath::Vector2(-1.0f, 0.0f);
	}

	tb_error("Error: Unexpected direction found.");
	return tbMath::Vector2::kZero;
}

//--------------------------------------------------------------------------------------------------------------------//

float LudumDare36::DirectionToOrienation(const LudumDare36::WorldDirection& direction)
{
	return tbMath::Convert::RadiansToDegrees(tbMath::ForwardVector2ToOrientation(DirectionToVector(direction)));
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldTransform::WorldTransform(int column, int row, const WorldDirection& direction) :
	mColumn(column),
	mRow(row),
	mDirection(direction),
	mPosition(static_cast<float>(LudumDare36::WorldEnvironment::kWorldTileSize * column) + (LudumDare36::WorldEnvironment::kWorldTileSize / 2.0f),
		static_cast<float>(LudumDare36::WorldEnvironment::kWorldTileSize * row) + (LudumDare36::WorldEnvironment::kWorldTileSize / 2.0f)),
	mForward(DirectionToVector(direction)),
	mOrientation(DirectionToOrienation(direction))
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldTransform::WorldTransform(int column, int row, const WorldDirection& direction, float explicitOrientation) :
	mColumn(column),
	mRow(row),
	mDirection(direction),
	mPosition(static_cast<float>(LudumDare36::WorldEnvironment::kWorldTileSize * column) + (LudumDare36::WorldEnvironment::kWorldTileSize / 2.0f),
		static_cast<float>(LudumDare36::WorldEnvironment::kWorldTileSize * row) + (LudumDare36::WorldEnvironment::kWorldTileSize / 2.0f)),
	mForward(DirectionToVector(direction)),
	mOrientation(explicitOrientation)
{
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldTransform LudumDare36::WorldTransform::Rotate90Left(const WorldTransform& input)
{
	WorldDirection newDirection(input.mDirection);

	switch (input.mDirection)
	{
	case WorldDirection::kNorth: newDirection = WorldDirection::kWest; break;
	case WorldDirection::kEast: newDirection = WorldDirection::kNorth; break;
	case WorldDirection::kSouth: newDirection = WorldDirection::kEast; break;
	case WorldDirection::kWest: newDirection = WorldDirection::kSouth; break;
	}

	return WorldTransform(input.mColumn, input.mRow, newDirection, input.mOrientation - 90.0f);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldTransform LudumDare36::WorldTransform::Rotate90Right(const WorldTransform& input)
{
	WorldDirection newDirection(input.mDirection);

	switch (input.mDirection)
	{
	case WorldDirection::kNorth: newDirection = WorldDirection::kEast; break;
	case WorldDirection::kEast: newDirection = WorldDirection::kSouth; break;
	case WorldDirection::kSouth: newDirection = WorldDirection::kWest; break;
	case WorldDirection::kWest: newDirection = WorldDirection::kNorth; break;
	}

	return WorldTransform(input.mColumn, input.mRow, newDirection, input.mOrientation + 90.0f);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldTransform LudumDare36::WorldTransform::Rotate180(const WorldTransform& input)
{
	WorldDirection newDirection(input.mDirection);

	switch (input.mDirection)
	{
	case WorldDirection::kNorth: newDirection = WorldDirection::kSouth; break;
	case WorldDirection::kEast: newDirection = WorldDirection::kWest; break;
	case WorldDirection::kSouth: newDirection = WorldDirection::kNorth; break;
	case WorldDirection::kWest: newDirection = WorldDirection::kEast; break;
	}

	return WorldTransform(input.mColumn, input.mRow, newDirection);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldTransform LudumDare36::WorldTransform::MoveWorld(const WorldTransform& input, const WorldDirection& direction, const int spaces)
{
	int columnChange(0);
	int rowChange(0);

	switch (direction)
	{
	case WorldDirection::kNorth: rowChange = -1; break;
	case WorldDirection::kEast: columnChange = 1; break;
	case WorldDirection::kSouth: rowChange = 1; break;
	case WorldDirection::kWest: columnChange = -1; break;
	}

	return WorldTransform(input.mColumn + (columnChange * spaces), input.mRow + (rowChange * spaces), input.mDirection, input.mOrientation);
}

//--------------------------------------------------------------------------------------------------------------------//

LudumDare36::WorldTransform LudumDare36::WorldTransform::MoveForward(const WorldTransform& input, int spaces)
{
	return MoveWorld(input, input.mDirection, spaces);
}

//--------------------------------------------------------------------------------------------------------------------//
